/*****************************************************************************
*
* Project Name : TB-UART Series GetEmissivity Code <softwareserial 이용>
* Version : 1.0 
* SYSTEM CLOCK : 16Mhz 
* BOARD : Arduino UNO. 5V operation 


 PORT Description

1. RX : 9           
2. TX : 8
  센서 전원은 3.3V로 하셔야 하며 포트 연결 방법은 회로도를 참고하십시오.
  센서 통신포트의 논리 레벨은 3.3V 이기 때문에 반드시 회로도를 참고하시기 바랍니다.

 Revision history.

1. 2019.5.20  : First version is released.
****************************************************************************/

#include <SoftwareSerial.h>

#define E_READ  0x05

SoftwareSerial mySerial(9, 8);                        // Rx 9, Tx 8

char _key;
uint8_t recv_cnt = 0;
uint8_t Timer1_Flag=0;
uint8_t TX_BUF[4];
uint16_t RX_BUF[8];                                        // RX buffer type : 2byte unsigned
int32_t  Sensor, Object;	                           // 부호 4byte 온도 저장 변수 

void setup() {
  pinMode(9,INPUT);
  pinMode(8,OUTPUT);
  Serial.begin(9600);
  while (!Serial) ;
  mySerial.begin(19200);
  while (!mySerial);
  delay(200);                             // wating for TB-UART setup time 

  Serial.println(" ");
  Serial.println("Send the 'y' key to read emissivity.");
  Serial.println(" ");
  while(1)
  {
    while(!Serial.available());  // 키값이 전송될 때 까지 대기
    _key = Serial.read();
    if (_key == 'y')
    {
        READ_CMD(E_READ);   
        break;
    }
  }
  
  while(1)
  {
    if(mySerial.available())
    { 
       RX_BUF[recv_cnt++] = mySerial.read();
       if(recv_cnt > 4)
       {
         recv_cnt = 0; break;
       }
    }
  } 
  if(Checksum_cal(5) == RX_BUF[3])
  {
    Serial.print("Stored Emissivity : ");
    Serial.println((float)RX_BUF[2]/100);
  }
  else
  {
    Serial.write("CHECKSUM Error");
  }  
}

void loop() {
}

void Transfer_Data(void)                                        
{
  unsigned char i;
  for(i=0; i<4; i++)
  {
    mySerial.write(TX_BUF[i]);
  }
}
unsigned char Checksum_cal(unsigned char CNT)                          // CHECKSUM 
{
  unsigned char count, SUM=0;
  for(count=0; count<CNT-2; count++)
  {
     SUM += RX_BUF[count];
  }
  return SUM;
}
void READ_CMD(unsigned char CMD)
{
  TX_BUF[0] = 0x11;
  TX_BUF[1] = 0x03;
  TX_BUF[2] = CMD;
  TX_BUF[3] = 0x98;
  
  Transfer_Data();
}
