/*****************************************************************************
*
* Project Name : TB-UART Series ReadTemp Code <softwareserial 이용>
* Version : 1.0 
* SYSTEM CLOCK : 16Mhz 
* BOARD : Arduino UNO. 5V operation 


 PORT Description

1. RX : 9           
2. TX : 8
  센서 전원은 3.3V로 하셔야 하며 포트 연결 방법은 회로도를 참고하십시오.
  센서 통신포트의 논리 레벨은 3.3V 이기 때문에 반드시 회로도를 참고하시기 바랍니다.

 Revision history.

1. 2019.5.20  : First version is released.
****************************************************************************/

#include <SoftwareSerial.h>

#define TEMP_READ  0x01

SoftwareSerial mySerial(9, 8);                        // Rx 9, Tx 8

uint8_t recv_cnt = 0;
uint8_t Timer1_Flag=0;
uint8_t TX_BUF[4];
uint16_t RX_BUF[8];                                        // RX buffer type : 2byte unsigned
int32_t  Sensor, Object;	                           // 부호 4byte 온도 저장 변수 

void setup() {
  pinMode(9,INPUT);
  pinMode(8,OUTPUT);
  Serial.begin(9600);
  while (!Serial) ;
  mySerial.begin(19200);
  while (!mySerial);
  delay(200);                             // wating for TB-UART setup time 
  Timer1_Init();                          // Timer1 setup : 100ms interval
  interrupts();                           // enable all interrupts
}

void loop() {
  if(Timer1_Flag)
  {
    Timer1_Flag = 0;
    READ_CMD(TEMP_READ);
    
    while(1)
    {
      if(mySerial.available())
      { 
         RX_BUF[recv_cnt++] = mySerial.read();
         if(recv_cnt>7)
         {
           recv_cnt = 0; break;
         }
      }
    } 
    if(Checksum_cal(8) == RX_BUF[6])
    {
      Object = RX_BUF[2]<<8 | RX_BUF[3];
      Sensor = RX_BUF[4]<<8 | RX_BUF[5];
      if(Object >40000)
      {
        Object -= 65536;
      }
      if(Sensor > 40000)
      {
        Sensor -= 65536;
      }
      
      Serial.print("Object Temp : ");
      Serial.print(float(Object)/100);
      Serial.print("    Sensor Temp : ");
      Serial.println(float(Sensor)/100);
    }
    else
    {
      Serial.write("CHECKSUM Error");
    }  
  }
     
}
ISR(TIMER1_OVF_vect)        // TimerCNT1 interrupt service routine 
{
  TCNT1 = 59286;            // preload timer : 65536-16MHz/256/10Hz  = 65536 - 16000000/256/10 = 59286
  Timer1_Flag = 1;
}
void Timer1_Init(void)
{
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 59286;            // preload timer : 65536-16MHz/256/10Hz  = 65536 - 16000000/256/10 = 59286
  TCCR1B |= (1 << CS12);    // 256 prescaler 
  TIMSK1 |= (1 << TOIE1);   // enable timer overflow interrupt
}
void Transfer_Data(void)                                        
{
  unsigned char i;
  for(i=0; i<4; i++)
  {
    mySerial.write(TX_BUF[i]);
  }
}
unsigned char Checksum_cal(unsigned char CNT)                          // CHECKSUM 
{
  unsigned char count, SUM=0;
  for(count=0; count<CNT-2; count++)
  {
     SUM += RX_BUF[count];
  }
  return SUM;
}
void READ_CMD(unsigned char CMD)
{
  TX_BUF[0] = 0x11;
  TX_BUF[1] = 0x03;
  TX_BUF[2] = CMD;
  TX_BUF[3] = 0x98;
  
  Transfer_Data();
}
